package freemind.modes;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;

import freemind.main.Resources;
import freemind.modes.MapAdapter;
import freemind.modes.MindMapNode;
import freemind.modes.mindmapmode.MindMapController;


public class Clone {

	private ArrayList nodes;
	private final MindMapController controller;
	private ImageIcon cloneIcon;
	
	public Clone(MindMapController controller, MindMapNode sourceNode, MindMapNode targetNode, String cloneID) {   
		this.controller = controller;
		nodes = new ArrayList();
		nodes.add(sourceNode.getObjectId(controller));  
		nodes.add(targetNode.getObjectId(controller)); 		
		sourceNode.setCloneID(cloneID);
		targetNode.setCloneID(cloneID);
		setCloneIcon(sourceNode, true);
		setCloneIcon(targetNode, true);
	}
	
	public Clone(MindMapController controller, MindMapNode node, String cloneID) {   
		this.controller = controller;
		nodes = new ArrayList();  
		nodes.add(node.getObjectId(controller));
		node.setCloneID(cloneID);
		setCloneIcon(node, true);
	}
	
	public void addNode(MindMapNode node, String cloneID){	
		String id = node.getObjectId(controller);
		if(!nodes.contains(id)){
			nodes.add(id);
			node.setCloneID(cloneID);
			setCloneIcon(node, true);
		}	
	}
	
	public void deleteNode(MindMapNode node, boolean removeLastClone){
		String id = node.getObjectId(controller);
		if(nodes.contains(id)){
			setCloneIcon(node, false);
			node.setCloneID(null);
			nodes.remove(id);
		}
		if(removeLastClone && hasOnlyOneClone()){
			clear();
		}
	}
	
	public String allToString(){
		String nodes = "[";
		String s = ", ";
		for(int i=0; i<this.nodes.size(); i++){
			if(i == this.nodes.size()-1) s = "]";
			nodes = nodes + this.nodes.get(i) + s;
		}
		return nodes;
	}
	
	public boolean hasOnlyOneClone(){
		if(nodes.size() == 1) return true;
		else return false;
	}
	
	public boolean isEmpty(){
		if(nodes.size() == 0) return true;
		else return false;
	}

	public ArrayList getNodes(){
		return nodes;
	}
	
	public String getNode(int i){
		return nodes.get(i).toString();
	}

	public void clear(){
		MindMapNode node;
		for(int i=0; i<nodes.size(); i++){
			node = controller.getNodeFromID(nodes.get(i).toString());
			setCloneIcon(node, false);
			remove(i, false);
		}
	}
	
	public void remove(int i, boolean removeLastClone){
		MindMapNode node = controller.getNodeFromID(nodes.get(i).toString());
		node.setCloneID(null);
		nodes.remove(i);
		if(removeLastClone && hasOnlyOneClone()){
			clear();
		}
	}
	
    private void setCloneIcon(MindMapNode node, boolean enabled) {
    	if(cloneIcon == null) {
			cloneIcon = new ImageIcon(controller.getResource("images/clone.png"));
		}	
        node.setStateIcon(this.getClass().getName(), (enabled) ? cloneIcon : null);
        controller.nodeChanged(node);
    }
    
}