/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2004  Joerg Mueller, Daniel Polansky, Christian Foltin and others.
 *
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Created on 25.08.2004
 */
/* $Id: CopySingleAction.java,v 1.1.2.1.2.4 2007/07/08 08:37:24 dpolivaev Exp $ */
package freemind.modes.mindmapmode.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;

import freemind.modes.mindmapmode.MindMapController;
import freemind.modes.MindMapNode;


//kris 09.08.07
public class PasteCloneAction extends AbstractAction {
	
	private final MindMapController pcMindMapController;
	public boolean isPastCloneAction; 
	public Vector pastedTree;
	
	public PasteCloneAction(MindMapController adapter) {
		 super(
		    adapter.getText("paste_clone"),
		    new ImageIcon(adapter.getResource("images/editclonepaste.png")));
		this.pcMindMapController = adapter;
		setEnabled(false);
		isPastCloneAction = false;
		pastedTree = new Vector();
	}

	public void actionPerformed(ActionEvent e) {
		isPastCloneAction = true;
		pcMindMapController.paste.actionPerformed(e); 
		isPastCloneAction = false;
	}
	
	public void createTreeOfClone(ArrayList source, MindMapNode pasted){
		setPastedTree(pasted);
		MindMapNode sourceNode;
		MindMapNode pastedNode;
		for(int i=0; i<source.size(); i++){	
			try{
				sourceNode = pcMindMapController.getNodeFromID(source.get(i).toString());
			}catch(IllegalArgumentException e){sourceNode = null;}	
			
			pastedNode = pcMindMapController.getNodeFromID(pastedTree.get(i).toString());
			
			if(sourceNode != null){
				pcMindMapController.clones.addClone(sourceNode, pastedNode);
				pcMindMapController.setNodeText(pastedNode, sourceNode.getText());
			}
		}
	}
	
	public void loadPastedTree(MindMapNode pasted){
		pastedTree.addElement(pasted.getObjectId(pcMindMapController));
		List children = pasted.getChildren();
		if(children.size() != 0){
	    	for(int i=0; i<children.size(); i++){
	    		MindMapNode child = (MindMapNode) children.get(i);
	    		if(child.hasChildren()){
	    			loadPastedTree(child);
	    		}else{  
	    			pastedTree.addElement(child.getObjectId(pcMindMapController));
	    		}
	    	}	
		}
	}
	
	public void setPastedTree(MindMapNode pasted){
		clearPastedTree();
		loadPastedTree(pasted);
	}
	
	public void clearPastedTree(){
    	pastedTree.clear();
	}

}